@@ -1,7 +1,7 @@
 from collections import defaultdict
 import json
 import time
-from visidata import colors, vd
+from visidata import colors, vd, clipdraw, ColorAttr, dispwidth
 
 __all__ = ['Animation', 'AnimationMgr']
 
@@ -13,11 +13,9 @@ class AttrDict(dict):
             if isinstance(v, dict):
                 v = AttrDict(v)
             return v
-        except AttributeError:
-            raise "eofo"
-        except KeyError:
+        except KeyError as e:
             if k.startswith("__"):
-                raise AttributeError
+                raise AttributeError from e
             return ''
     def __setattr__(self, k, v):
         self[k] = v
@@ -33,24 +31,38 @@ class Animation:
         self.width = 0
         self.load_from(fp)
 
-    def iterdeep(self, rows, x=0, y=0, parents=None):
-        'Walk rows deeply and generate (row, x, y, [ancestors]) for each row.'
+    def iterdeep(self, rows, x=0, y=0, parents=None, **kwargs):
+        'Walk rows deeply and generate (row, x, y, [ancestors]) for each row, filtering on kwargs.'
         for r in rows:
             newparents = (parents or []) + [r]
             if r.type == 'frame': continue
             if r.ref:
                 assert r.type == 'ref'
                 g = self.groups[r.ref]
-                yield from self.iterdeep(map(AttrDict, g.rows or []), x+r.x, y+r.y, newparents)
+                if self.matches(r, kwargs):
+                    yield from self.iterdeep(map(AttrDict, g.rows or []), x+r.x, y+r.y, newparents)
             else:
-                yield r, x+r.x, y+r.y, newparents
-                yield from self.iterdeep(map(AttrDict, r.rows or []), x+r.x, x+r.y, newparents)
+                if self.matches(r, kwargs):
+                    yield r, x+r.x, y+r.y, newparents
+                    yield from self.iterdeep(map(AttrDict, r.rows or []), x+r.x, x+r.y, newparents)
+
+    def matches(self, row, values):
+        for k, v in values.items():
+            actualv = getattr(row, k, None)
+            if isinstance(actualv, (list, tuple)) and isinstance(v, (list, tuple)):
+                if not any(x in actualv for x in v):
+                    return False
+            elif v != actualv:
+                return False
+        return True
 
     def load_from(self, fp):
         for line in fp.readlines():
             r = AttrDict(json.loads(line))
             if r.type == 'frame':
-                self.frames[r.id].update(r)
+                f = self.frames[r.id]
+                f.update(r)
+                f.rows = []
             elif r.type == 'group':
                 self.groups[r.id].update(r)
 
@@ -65,12 +77,15 @@ class Animation:
             self.total_ms = sum(f.duration_ms or 0 for f in self.frames.values())
             for f in self.frames.values():
                 for r, x, y, _ in self.iterdeep(f.rows):
-                    self.width = max(self.width, x+len(r.text))
+                    self.width = max(self.width, x+dispwidth(r.text))
                     self.height = max(self.height, y)
 
-    def draw(self, scr, *, t=0, x=0, y=0, loop=False, **kwargs):
-        for r, dx, dy, _ in self.iterdeep(self.frames[''].rows):
-            scr.addstr(y+dy, x+dx, r.text, colors[r.color])
+    def draw(self, scr, *, t=0, x=0, y=0, loop=False, attr=ColorAttr(), **kwargs):
+        windowHeight, windowWidth = scr.getmaxyx()
+        for r, dx, dy, _ in self.iterdeep(self.frames[''].rows, **kwargs):
+            text = f'[:onclick {r.href}]{r.text}[/]' if r.href else r.text
+            if y+dy < windowHeight:
+                clipdraw(scr, y+dy, x+dx, text, attr.update(colors[r.color], 2))
 
         if not self.total_ms:
             return None
@@ -79,8 +94,10 @@ class Animation:
         for f in self.frames.values():
             ms -= int(f.duration_ms or 0)
             if ms < 0:
-                for r, dx, dy, _ in self.iterdeep(f.rows):
-                    scr.addstr(y+dy, x+dx, r.text, colors[r.color])
+                for r, dx, dy, _ in self.iterdeep(f.rows, **kwargs):
+                    text = f'[:onclick {r.href}]{r.text}[/]' if r.href else r.text
+                    if y+dy < windowHeight:
+                        clipdraw(scr, y+dy, x+dx, text, colors[r.color])
 
                 return -ms/1000
 
@@ -119,9 +136,9 @@ class AnimationMgr:
         for row in self.active:
             startt, anim, akwargs = row
             kwargs.update(akwargs)
-            nextt = anim.draw(scr, t=t-startt, **kwargs)
+            nextt = anim.draw(scr, t=t+startt, **kwargs)
             if nextt is None:
-                if not akwargs.get('loop'):
+                if not akwargs.get('loop', True):
                     done.append(row)
             else:
                 times.append(t+nextt)
