@@ -8,7 +8,7 @@ class Extensible:
 
     @classmethod
     def init(cls, membername, initfunc=lambda: None, copy=False):
-        'Append equivalent of ``self.<membername> = initfunc()`` to ``<cls>.__init__``.  If *copy* is True, <membername> will be copied when object is copied.'
+        'Prepend equivalent of ``self.<membername> = initfunc()`` to ``<cls>.__init__``.  If *copy* is True, <membername> will be copied when object is copied.'
 
         def thisclass_hasattr(cls, k):
             return getattr(cls, k, None) is not getattr(cls.__bases__[0], k, None)
@@ -16,12 +16,12 @@ class Extensible:
         # must check hasattr first or else this might be parent's __init__
         oldinit = thisclass_hasattr(cls, '__init__') and getattr(cls, '__init__')
         def newinit(self, *args, **kwargs):
+            if not hasattr(self, membername):  # can be overridden by a subclass
+                setattr(self, membername, initfunc())
             if oldinit:
                 oldinit(self, *args, **kwargs)
             else:
                 super(cls, self).__init__(*args, **kwargs)
-            if not hasattr(self, membername):  # can be overridden by a subclass
-                setattr(self, membername, initfunc())
         cls.__init__ = wraps(oldinit)(newinit) if oldinit else newinit
 
         oldcopy = thisclass_hasattr(cls, '__copy__') and getattr(cls, '__copy__')
@@ -34,12 +34,23 @@ class Extensible:
             return ret
         cls.__copy__ = wraps(oldcopy)(newcopy) if oldcopy else newcopy
 
+    @classmethod
+    def superclasses(cls):
+        yield cls
+        yield from cls.__bases__
+        for b in cls.__bases__:
+            if hasattr(b, 'superclasses'):
+                yield from b.superclasses()
+
     @classmethod
     def api(cls, func):
         oldfunc = getattr(cls, func.__name__, None)
         if oldfunc:
             func = wraps(oldfunc)(func)
+        from visidata import vd
+        func.importingModule = vd.importingModule
         setattr(cls, func.__name__, func)
+        func._extensible_api = True
         return func
 
     @classmethod
@@ -75,10 +86,19 @@ class Extensible:
 
     @classmethod
     def class_api(cls, func):
+        '''`@Class.class_api` works much like `@Class.api`, but for class methods. This method is used internally but may not be all that useful for plugin and module authors. Note that `@classmethod` must still be provided, and **the order of multiple decorators is crucial**, in that `@<class>.class_api` must come before `@classmethod`:
+
+::
+        @Sheet.class_api
+        @classmethod
+        def addCommand(cls, ...):
+        '''
+
         name = func.__get__(None, dict).__func__.__name__
         oldfunc = getattr(cls, name, None)
         if oldfunc:
             func = wraps(oldfunc)(func)
+        func._extensible_api = True
         setattr(cls, name, func)
         return func
 
@@ -89,19 +109,22 @@ class Extensible:
         def dofunc(self):
             return func(self)
         setattr(cls, func.__name__, dofunc)
+        func._extensible_api = True
         return dofunc
 
     @classmethod
     def lazy_property(cls, func):
         'Return ``func()`` on first access and cache result; return cached result thereafter.'
+        name = '_' + func.__name__
+        cls.init(name, lambda: None, copy=False)
         @property
         @wraps(func)
         def get_if_not(self):
-            name = '_' + func.__name__
-            if not hasattr(self, name):
+            if getattr(self, name, None) is None:
                 setattr(self, name, func(self))
             return getattr(self, name)
         setattr(cls, func.__name__, get_if_not)
+        func._extensible_api = True
         return get_if_not
 
     @classmethod
