@@ -40,7 +40,7 @@ class MacroSheet(IndexSheet):
     def putChanges(self):
         self.commitDeletes()  #1569  apply deletes early for saveSheets below
 
-        vd.saveSheets(self.source, self, confirm_overwrite=False)
+        vd.sync(vd.saveSheets(self.source, self, confirm_overwrite=False))
         self._deferredDels.clear()
         self.reload()
 
@@ -107,9 +107,10 @@ def afterExecSheet(cmdlog, sheet, escaped, err):
     if not vd.activeCommand: return
     if vd.activeCommand.longname == 'macro-record': return
 
-    cmd = copy(vd.activeCommand)
-    cmd.sheet = ''
-    vd.macroMode.addRow(cmd)
+    if vd.activeCommand.replayable:
+        cmd = copy(vd.activeCommand)
+        cmd.sheet = ''
+        vd.macroMode.addRow(cmd)
 
 
 @CommandLogJsonl.api
