@@ -53,7 +53,7 @@ class AggrColumn(Column):
     def calcValue(col, row):
         if col.sheet.loading:
             return visidata.INPROGRESS
-        return col.aggregator(col.origCol, row.sourcerows)
+        return col.aggregator.aggregate(col.origCol, row.sourcerows)
 
 
 def makeAggrColumn(aggcol, aggregator):
@@ -175,13 +175,13 @@ class PivotSheet(Sheet):
                         c = Column(colname,
                                     type=aggregator.type or aggcol.type,
                                     aggvalue=value,
-                                    getter=lambda col,row,aggcol=aggcol,agg=aggregator: agg(aggcol, row.pivotrows.get(col.aggvalue, [])))
+                                    getter=lambda col,row,aggcol=aggcol,agg=aggregator: agg.aggregate(aggcol, row.pivotrows.get(col.aggvalue, [])))
                         self.addColumn(c)
 
 #                    if aggregator.name != 'count':  # already have count above
 #                        c = Column('Total_' + aggcol.name,
 #                                    type=aggregator.type or aggcol.type,
-#                                    getter=lambda col,row,aggcol=aggcol,agg=aggregator: agg(aggcol, row.sourcerows))
+#                                    getter=lambda col,row,aggcol=aggcol,agg=aggregator: agg.aggregate(aggcol, row.sourcerows))
 #                        self.addColumn(c)
 
     @asyncthread
@@ -291,8 +291,11 @@ class PivotSheet(Sheet):
 @PivotSheet.api
 def addcol_aggr(sheet, col):
     hasattr(col, 'origCol') or vd.fail('not an aggregation column')
-    for agg in vd.chooseAggregators():
-        sheet.addColumnAtCursor(makeAggrColumn(col.origCol, vd.aggregators[agg]))
+    for agg_choice in vd.chooseAggregators():
+        agg_or_list = vd.aggregators[agg_choice]
+        aggs = agg_or_list if isinstance(agg_or_list, list) else [agg_or_list]
+        for agg in aggs:
+            sheet.addColumnAtCursor(makeAggrColumn(col.origCol, vd.aggregators[agg]))
 
 
 Sheet.addCommand('W', 'pivot', 'vd.push(makePivot(sheet, keyCols, [cursorCol]))', 'open Pivot Table: group rows by key column and summarize current column')
