@@ -20,13 +20,13 @@ class InProgress(Exception):
 
 INPROGRESS = TypedExceptionWrapper(None, exception=InProgress())  # sentinel
 
-vd.option('col_cache_size', 0, 'max number of cache entries in each cached column', max_help=-1)
-vd.option('disp_formatter', 'generic', 'formatter to create the text in each cell (also used by text savers)', replay=True, max_help=0)
-vd.option('disp_displayer', 'generic', 'displayer to render the text in each cell', replay=False, max_help=0)
+vd.option('col_cache_size', 0, 'max number of cache entries in each cached column')
+vd.option('disp_formatter', 'generic', 'formatter to create the text in each cell (also used by text savers)', replay=True)
+vd.option('disp_displayer', 'generic', 'displayer to render the text in each cell', replay=False)
 
 
 class DisplayWrapper:
-    def __init__(self, value=None, *, typedval=None, text=None, note=None, notecolor=None, error=None):
+    def __init__(self, value=None, *, typedval=None, text='', note=None, notecolor=None, error=None):
         self.value = value      # actual value (any type)
         self.typedval = typedval  # consistently typed value (or None)
         self.text = text  # displayed string
@@ -40,21 +40,11 @@ class DisplayWrapper:
     def __eq__(self, other):
         return self.value == other
 
-def _default_colnames():
-    'A B C .. Z AA AB .. ZZ AAA .. to infinity'
-    i=0
-    while True:
-        i += 1
-        for x in itertools.product(string.ascii_uppercase, repeat=i):
-            yield ''.join(x)
-
-default_colnames = _default_colnames()
-
 
 class Column(Extensible):
     '''Base class for all column types.
 
-        - *name*: name of this column.
+        - *name*: name of this column; if None, current sheet will assign a name
         - *type*: ``anytype str int float date`` or other type-like conversion function.
         - *cache*: cache behavior
 
@@ -71,7 +61,10 @@ class Column(Extensible):
     def __init__(self, name=None, *, type=anytype, cache=False, **kwargs):
         self.sheet = ExplodingMock('use addColumn() on all columns')  # owning Sheet, set in .recalc() via Sheet.addColumn
         if name is None:
-            name = next(default_colnames)
+            if vd.sheet: # get a column name from the current sheet
+                name = vd.sheet.incremented_colname()
+            else:
+                name = ''
         self.name = str(name) # display visible name
         self.fmtstr = ''      # by default, use str()
         self._type = type     # anytype/str/int/float/date/func
@@ -86,7 +79,7 @@ class Column(Extensible):
         self.formatter = ''
         self.displayer = ''
         self.defer = False
-        self.max_help = 10    # auto-hide above this disp_help level
+        self.disp_expert = 0    # do not show if 'nometacols' in options.disp_help_flags
 
         self.setCache(cache)
         for k, v in kwargs.items():
@@ -243,12 +236,16 @@ class Column(Extensible):
         return self.make_formatter()(*args, **kwargs)
 
     def formatValue(self, typedval, width=None):
-        'Return displayable string of *typedval* according to ``Column.fmtstr``.'
+        '''Return displayable string of *typedval* according to ``Column.fmtstr``.
+        If *width* is not None, values are clipped to that width when *typedval*
+        is a dict/list/tuple, but not for other types.'''
         if typedval is None:
             return None
 
         if self.type is anytype:
             if isinstance(typedval, (dict, list, tuple)):
+                if width is None:
+                    return ''.join(iterchars(typedval))
                 dispval, dispw = clipstr(iterchars(typedval), width)
                 return dispval
 
@@ -264,7 +261,9 @@ class Column(Extensible):
 
            The 'generic' displayer does not do any formatting.
         '''
-        if width is not None and width > 1 and vd.isNumeric(self):
+        if width is not None and width > 1 and \
+                vd.isNumeric(self) and \
+                isinstance(dw.typedval, (int, float)):
             yield ('', dw.text.rjust(width-2))
         else:
             yield ('', dw.text)
@@ -276,7 +275,7 @@ class Column(Extensible):
            The 'full' displayer allows formatting like [:color].
         '''
         if width is not None and width > 1 and vd.isNumeric(self):
-            yield from iterchunks(text.rjust(width-2))
+            yield from iterchunks(dw.text.rjust(width-2))
         else:
             yield from iterchunks(dw.text)
 
@@ -355,7 +354,8 @@ class Column(Extensible):
         return ret
 
     def getCell(self, row):
-        'Return DisplayWrapper for displayable cell value.'
+        '''Return DisplayWrapper for displayable cell value.
+        For dict/list/tuple cells, the width of the value returned is capped at the column width.'''
         cellval = wrapply(self.getValue, row)
         typedval = wrapply(self.type, cellval)
 
@@ -368,7 +368,7 @@ class Column(Extensible):
                     dispval = options.disp_error_val
                 return DisplayWrapper(cellval.val, error=exc.stacktrace,
                                         text=dispval,
-                                        note=options.note_getter_exc,
+                                        note=f'[:onclick error-cell]{options.disp_note_getexc}[:]',
                                         notecolor='color_error')
             elif typedval.val is None:  # early out for strict None
                 return DisplayWrapper(None, text='',  # force empty display for None
@@ -377,25 +377,24 @@ class Column(Extensible):
             elif isinstance(typedval, TypedExceptionWrapper):  # calc succeeded, type failed
                 return DisplayWrapper(typedval.val, text=str(cellval),
                                             error=typedval.stacktrace,
-                                            note=options.note_type_exc,
+                                            note=f'[:onclick error-cell]{options.disp_note_typeexc}[:]',
                                             notecolor='color_warning')
             else:
                 return DisplayWrapper(typedval.val, text=str(typedval.val),
-                                            error='unknown',
-                                            note=options.note_type_exc,
+                                            error=['unknown'],
+                                            note=f'[:onclick error-cell]{options.disp_note_typeexc}[:]',
                                             notecolor='color_warning')
-
         elif isinstance(typedval, threading.Thread):
             return DisplayWrapper(None,
                                 text=options.disp_pending,
-                                note=options.note_pending,
+                                note=options.disp_note_pending,
                                 notecolor='color_note_pending')
 
         dw = DisplayWrapper(cellval)
         dw.typedval = typedval
 
         try:
-            dw.text = self.format(typedval, width=(self.width or 0)*2) or ''
+            dw.text = self.format(typedval, width=self.width) or ''
 
             # annotate cells with raw value type in anytype columns, except for strings
             if self.type is anytype and type(cellval) is not str:
@@ -411,14 +410,15 @@ class Column(Extensible):
                 dw.text = str(cellval)
             except Exception as e:
                 dw.text = str(e)
-            dw.note = options.note_format_exc
+            dw.note = options.disp_note_fmtexc
             dw.notecolor = 'color_warning'
 
-#        dw.display = self.display(dw)   # set during draw() when colwidth is known
+#        dw.display = self.display(dw)   # set during draw() once colwidth is known
         return dw
 
     def getDisplayValue(self, row):
-        'Return string displayed in this column for given *row*.'
+        '''Return string displayed in this column for given *row*.
+        For dict/list/tuple cells, the width of the display value returned is capped at the column width.'''
         return self.getCell(row).text
 
     def putValue(self, row, val):
@@ -446,6 +446,7 @@ class Column(Extensible):
         self.recalc()
         return vd.status('set %d cells to %d values' % (len(rows), len(values)))
 
+    @asyncthread
     def setValuesTyped(self, rows, *values):
         'Set values on this column for *rows* to *values*, coerced to column type, recycling values as needed to fill *rows*.  Abort on type exception.'
         vd.addUndoSetValues([self], rows)
@@ -457,22 +458,36 @@ class Column(Extensible):
         return vd.status('set %d cells to %d values' % (len(rows), len(values)))
 
     def getMaxWidth(self, rows):
-        'Return the maximum length of any cell in column or its header (up to window width).'
-        w = 0
+        'Return the maximum length of any cell in column or its header (up to drawable window width).'
+        drawable_width = self.sheet.windowWidth-1
         nlen = dispwidth(self.name)
-        if len(rows) > 0:
-            w_max = 0
-            for r in rows:
-                row_w = dispwidth(self.getDisplayValue(r), maxwidth=self.sheet.windowWidth)
-                if w_max < row_w:
-                    w_max = row_w
-                if w_max >= self.sheet.windowWidth:
-                    break  #1747  early out to speed up wide columns
-            w = w_max
-        w = max(w, nlen)+2
-        w = min(w, self.sheet.windowWidth)
-        return w
+        w_max = nlen
+        for r in rows:
+            row_w = self.measureValueWidthCapped(r, maxwidth=drawable_width)
+            if w_max < row_w:
+                w_max = row_w
+            if w_max >= self.sheet.windowWidth:
+                break  #1747  early out to speed up wide columns
+        return min(w_max+2, drawable_width)
 
+    def measureValueWidthCapped(self, row, maxwidth=None):
+        '''Measure the width of the contents of a cell. Stop measuring at *maxwidth*,
+           to save time iterating over very long dict/list/tuple values.
+           If *maxwidth* is None, return the full width.'''
+        # The value classification logic here is taken from getCell,
+        # modified to cap the width examined for any dict/list/tuple
+        cellval = wrapply(self.getValue, row)
+        typedval = wrapply(self.type, cellval)
+        if isinstance(typedval, (TypedWrapper, threading.Thread)):
+            return dispwidth(self.getCell(row).text, maxwidth=maxwidth)
+        try:
+            text = self.format(typedval, width=maxwidth) or ''
+        except Exception as e:  # formatting failure
+            try:
+                text = str(cellval)
+            except Exception as e:
+                text = str(e)
+        return dispwidth(text, maxwidth=maxwidth)
 
 
 # ---- basic Columns
@@ -537,39 +552,6 @@ def SubColumnItem(idx, c, **kwargs):
         kwargs['name'] = c.name
     return SubColumnFunc(origcol=c, subfunc=getitemdef, expr=idx, **kwargs)
 
-class ExprColumn(Column):
-    'Column using *expr* to derive the value from each row.'
-    def __init__(self, name, expr=None, **kwargs):
-        super().__init__(name, **kwargs)
-        self.expr = expr or name
-        self.ncalcs = 0
-        self.totaltime = 0
-        self.maxtime = 0
-
-    def calcValue(self, row):
-        t0 = time.perf_counter()
-        r = self.sheet.evalExpr(self.compiledExpr, row, col=self)
-        t1 = time.perf_counter()
-        self.ncalcs += 1
-        self.maxtime = max(self.maxtime, t1-t0)
-        self.totaltime += (t1-t0)
-        return r
-
-    def putValue(self, row, val):
-        a = self.getDisplayValue(row)
-        b = self.format(self.type(val))
-        if a != b:
-            vd.warning("Cannot change value of calculated column.  Use `'` to freeze column.")
-
-    @property
-    def expr(self):
-        return self._expr
-
-    @expr.setter
-    def expr(self, expr):
-        self.compiledExpr = compile(expr, '<expr>', 'eval') if expr else None
-        self._expr = expr
-
 
 class SettableColumn(Column):
     'Column using rowid to store and retrieve values internally.'
@@ -592,7 +574,6 @@ vd.addGlobals(
     getitemdef=getitemdef,
     AttrColumn=AttrColumn,
     ItemColumn=ItemColumn,
-    ExprColumn=ExprColumn,
     SettableColumn=SettableColumn,
     SubColumnFunc=SubColumnFunc,
     SubColumnItem=SubColumnItem,
@@ -601,6 +582,5 @@ vd.addGlobals(
 # synonyms
     ColumnItem=ItemColumn,
     ColumnAttr=AttrColumn,
-    ColumnExpr=ExprColumn,
     DisplayWrapper=DisplayWrapper
 )
